/*
 CanDume.js Ver 1.8 (2023-05-*04)
  by KANEKO Kiyoshi / shogi-problem.org & tsume-kobo.org 2019-2023 / CC-BY

            Ver 1.8 (2023-05-04)
            Ver 1.7 (2021-08-29)
            Ver 1.6 (2021-06-27)
            Ver 1.5 (2021-04-29)
            Ver 1.4 (2021-04-24)
            Ver 1.3 (2021-02-06)
            Ver 1.2 (2021-01-10)
            Ver 1.1 (2021-01-05)
            Ver 1.0 (2020-12-20)

 このソースコードは、Google Closure Compiler（GCC）で処理する前提で作成している。
 （このままコンパイルしなくても動作はするはず）
 Google Closure Compiler:  https://developers.google.com/closure/compiler
 ツール作成に使用したコンパイルオプション：
  --compilation_level ADVANCED_OPTIMIZATIONS --language_in ECMASCRIPT_NEXT --language_out ECMASCRIPT_2020
*/
// ==ClosureCompiler==
// @compilation_level ADVANCED_OPTIMIZATIONS
// @language_in ECMASCRIPT_NEXT
// @language_out ECMASCRIPT_2020
// @output_file_name candume.js
// ==/ClosureCompiler==

"use strict";

(() => { // GCCがvarを生成するため全体をクロージャにしておく
    const isSameSizeCanvas = true; // GCC用コンパイルスイッチ
                                   // trueなら全てのcanvasタグ（data-fileがあるもの）を全て同じパラメータ処理する
                                   // falseならcanvasサイズは個別設定で、CSSのほか、canvasのwidth属性、height属性も有効
    const isTrustInFile = true; // GCC用コンパイルスイッチ
                                // trueなら入力ファイルを信頼してエラー処理を省略する
                                // falseならCSA書式レベルのエラー処理をする（将棋のルール的チェックはナシ）

    // 雑用定数いろいろ
    const num9to1 = [9, 8, 7, 6, 5, 4, 3, 2, 1]; // 盤面のループで使う
    const pxUnit = "px";
    const isUndefined = (p) => ("undefined" == (typeof p));
    const strictNumber = (p) => ((!p || /\D/.test(p)) ? 0 : Number(p)); // 数字だけの文字列のみ十進整数変換
    const docCurrentScript = document.currentScript;
    const docTarget = docCurrentScript.parentNode; // Ver.1,4 scriptを含むタグ内のみ処理対象とする

    // script要素、canvas要素のパラメータ名称
    const currentScriptDataset = docCurrentScript.dataset;
    const dataFile = "file"; // 入力ファイル
    const dataZero = "zero"; // ０手目の表示（○手詰の代わりに表示する文字列）
    const dataStart = "start"; // 初期状態
    const dataWatermark = "wm"; // Watermarkの設定（Ver1.1で追加）右クリックで表示するウォータマーク文字列

    // 盤面座標文字（関数の外に置いて多少のメモリ節約）
    const numberA = ["", "１", "２", "３", "４", "５", "６", "７", "８", "９"];
    const numberJ = ["", "一", "二", "三", "四", "五", "六", "七", "八", "九"];

    // 駒関係の定数（関数の外に置いて多少のメモリ節約）
    const pieceCharJ = [ // 盤面の駒文字
        //0     1     2     3     4     5     6     7
        "歩", "香", "桂", "銀", "金", "角", "飛", "玉", // 0x00
        "と", "杏", "圭", "全",   "", "馬", "龍",   "", // 0x08
        "歩", "香", "桂", "銀", "金", "角", "飛", "王", // 0x10
        "と", "杏", "圭", "全",   "", "馬", "龍",   "", // 0x18
        "歩", "香", "桂", "銀", "金", "角", "飛", "玉", // 0x20
        "と", "杏", "圭", "全",   "", "馬", "龍"        // 0x28
    ];
    const pieceCharCSA = [ // csaファイルを読み込むときの駒文字
        //0      1      2      3      4      5      6      7
         "FU",  "KY",  "KE",  "GI",  "KI",  "KA",  "HI",  "OU", // 0x00
         "TO",  "NY",  "NK",  "NG",    "",  "UM",  "RY",    "", // 0x08
        "+FU", "+KY", "+KE", "+GI", "+KI", "+KA", "+HI", "+OU", // 0x10
        "+TO", "+NY", "+NK", "+NG",    "", "+UM", "+RY",    "", // 0x18
        "-FU", "-KY", "-KE", "-GI", "-KI", "-KA", "-HI", "-OU", // 0x20
        "-TO", "-NY", "-NK", "-NG",    "", "-UM", "-RY"         // 0x28
    ];
    const pieceCount = 8; // 駒の種類
    const bitMaskOffence = 0x10; //  9ビット目は攻め方
    const bitMaskDefense = 0x20; // 10ビット目は受け方

    // ダークテーマ関係まとめ。compeと部品共通化（V1.8）
    const colorControls = (() => {
        const elementsDrawList = [];
        const colorObj = { setElement: (elem) => elementsDrawList.push(elem.option.fullDraw) };
        const darkThemeQuery = window.matchMedia("(prefers-color-scheme: dark)");
        const onChangeDark = () => {
            const isDark = darkThemeQuery.matches;
            colorObj.fontColor = (isDark ? "white" : "black"); // 盤駒文字の色　デフォルト黒　ダークモードで白
            colorObj.destinationColor = (isDark ? "maroon" : "yellow"); // 着手地点の色　デフォルト黄色 ダークモードでマルーン
            for (const item of elementsDrawList) {
                item();
            }
        };
        darkThemeQuery.addEventListener("change", onChangeDark);
        onChangeDark();
        return colorObj;
    })();

    // イベント関係まとめて設定
    const setEvent = (() => {
        const timerMax = 1000; // 連続移動の間隔ミリ秒 最大値（遅い）
        const timerMin = 200;  // 連続移動の間隔ミリ秒 最小値（速い）
        const timerRate = 1.2; // 連続移動の加速率

        let currentElem = {}; // emptyObject
        let timerId; // timerをクリアするとき使う
        let stepIncDec = 0;
        let isAutoRun = false; // 連続移動が作動し始めたときtrue
        let isDisableMouse = false; // touch中のマウス移動を無効化するフラグ
        let isDisableTouchEnd = true; // touchMove後のtouchEndを無効化するフラグ

        const tryCancel = (e) => { // cancel可能なイベントeをキャンセルする
            e.stopPropagation();
            if (e.cancelable) {
                e.preventDefault();
                return false;
            } else {
                return true; // 都合上、cancel失敗をtrueに
            }
        };

        const setIncDec = () => currentElem.option.setIncDec(stepIncDec);

        const startAutoRun = (incDec, isFirstMove) => { // 連続移動開始　isFirstMove=trueなら１手動かす
            let timerNext = timerMax;
            stepIncDec = incDec;
            isAutoRun = false;
            const autoRun = () => {
                isAutoRun = setIncDec(); // setIncDec成功のときisAutoRunをセット
                if (isAutoRun) {
                    timerNext /= timerRate; // 加速
                    if (timerNext < timerMin) {
                        timerNext = timerMin;
                    }
                    timerId = setTimeout(autoRun, timerNext);
                }
            };
            if (isFirstMove) {
                setIncDec();
            }
            timerId = setTimeout(autoRun, timerMax);
        };

        const stopAutoRun = (isLastMove) => { // 連続移動終了　isLastMove=trueなら１手動かす
            clearTimeout(timerId);
            isAutoRun = false;
            if (isLastMove) {
                setIncDec();
            }
        };

        const onMouseDown = (e) => {
            if (tryCancel(e) || isDisableMouse || e.button) { // tryCancelは先にチャレンジする。e.button != 0 と同義
                isDisableMouse = false;
                return;
            }
            stopAutoRun(false);
            if (currentElem == e.target) {
                const incDec = currentElem.option.incDecByPoint(e);
                startAutoRun(incDec, false);
            }
        };

        const onMouseUp = (e) => {
            if (tryCancel(e) || isDisableMouse || e.button) { // tryCancelは先にチャレンジする。e.button != 0 と同義
                isDisableMouse = false;
                return;
            }
            if (currentElem != e.target) {
                e.target.focus();
            } else if (!isAutoRun) {
                stopAutoRun(true); // autoRunでなければ１手動かす
            }
        };

        const onTouchStart = (e) => { // スクロール優先につきtryCancel禁止
            stopAutoRun(false);
            isDisableMouse = true; // マウスイベント延焼防止
            isDisableTouchEnd = false;
            const tcs = e.touches;
            if (tcs.length != 1) {
                return; // 指１歩のみ処理対象
            }
            if (currentElem == e.target) {
                const incDec = currentElem.option.incDecByPoint(tcs[0]);
                startAutoRun(incDec, false);
            }
        };

        const onTouchEnd = (e) => {
            isDisableMouse = true; // マウスイベント延焼防止
            if (tryCancel(e) || isDisableTouchEnd || e.touches.length) { // tryCancelは先にチャレンジする
                return; // e.touches.length > 0 と同義→指０本のみ処理対象
            }
            if (currentElem != e.target) {
                e.target.focus();
            } else if (!isAutoRun) { // autoRunでなければ１手動かす
                stopAutoRun(true);
            }
        };

        const onTouchMove = (e) => { // スクロール優先につきtryCancel禁止
            isDisableTouchEnd = true; // touchMoveしたらタッチの選択を無効化
            stopAutoRun(false);
        };

        const onKeyDown = (e) => {
            if (e.repeat) {
                return; // オートリピートの抑止
            }
            stopAutoRun(false);
            e.target.focus(); // focusされていないケースの対応
            if (e.key == "ArrowLeft") {
                startAutoRun(-1, true); // キーボードは押した時点で１手動かす
            } else if (e.key == "ArrowRight") {
                startAutoRun(1, true); // キーボードは押した時点で１手動かす
            }
        };

        const onKeyUp = (e) => stopAutoRun(false); // キーボードは離した時点でストップ

        const isTouchDevice = ("ontouchstart" in document); // タッチデバイスの判定
        const watermarkString = (isTouchDevice ? "" : (currentScriptDataset[dataWatermark] ?? location.hostname));
        const _setWatermarkString = (e) => e.target.option.setWatermarkString(watermarkString);
        const onContextMenu = (watermarkString ? _setWatermarkString : tryCancel); // watermarkString==""ならcontextMenuをtryCancel

        const onFocus = (e) => {
            currentElem = e.target; // Ver1.7でバグ修正。空ElementでのAutoRunを防止
            currentElem.scrollIntoView({
                behavior: "smooth",
                block: "center",
                inline: "nearest"
            });
        };

        const onBlur = (e) => {
            stopAutoRun(false); // Ver1.7でバグ修正。空ElementでのAutoRunを防止
            currentElem = {};
        };

        return (elem) => {
            elem.addEventListener("mousedown", onMouseDown);
            elem.addEventListener("mouseup", onMouseUp);
            elem.addEventListener("keydown", onKeyDown);
            elem.addEventListener("keyup", onKeyUp);
            if (isTouchDevice) {
                elem.addEventListener("touchstart", onTouchStart, { passive: true });
                elem.addEventListener("touchend", onTouchEnd);
                elem.addEventListener("touchmove", onTouchMove, { passive: true });
            }
            elem.addEventListener("contextmenu", onContextMenu);
            elem.addEventListener("selectstart", tryCancel); // スマホ版Chromeでの親要素へのbubbling抑止（Ver1.2で追加）
            // elem.addEventListener("click", tryCancel); // 今のところ無くてもOK
            // elem.addEventListener("dblclick", tryCancel); // 今のところ無くてもOK
            elem.addEventListener("focus", onFocus);
            elem.addEventListener("blur", onBlur);
            if (elem["tabIndex"] < 0) { // tabindexが設定されていれば尊重する
                elem["tabIndex"] = 0; // canvas選択を可能にするため
            }
        }
    })();

    const candume = (myElement, myPositionSet) => {

        let nowMoves = 0;
        let maxMoves = 0;
        let zeroGuide = "";
        let myWatermarkString = "";
        let myBoard = [];
        let myHand = [];
        let myDestinationX = [];
        let myDestinationY = [];

        myElement.width = myPositionSet.fullWidth;
        myElement.height = myPositionSet.fullHeight;
        myElement.style.width = myPositionSet.fullWidth + pxUnit;
        myElement.style.height = myPositionSet.fullHeight + pxUnit;

        const num7to0 = [7, 6, 5, 4, 3, 2, 1, 0]; // 持駒のループで使う

        const elemDataset = myElement.dataset;
        const fileName = elemDataset[dataFile]; // undefinedは除去されている前提

        const context = myElement.getContext("2d");

        const boardIndex = (te, x, y) => (te * 9 * 9) + (y * 9) - x;
        const handIndex = (te, k) => (te * pieceCount) + toPieceOriginal(k);

        const noPiece = (-1);
        const getCodeByCSA = (csaChr) => pieceCharCSA.indexOf(csaChr); // アンマッチの場合は-1（noPiece）

        const toOffence = (c) => (c | bitMaskOffence);
        const toDefense = (c) => (c | bitMaskDefense);

        const isOffence = (c) => !(c & bitMaskDefense); // defenseビット0でoffenceチェック。!x は x==0と同義
        const isDefense = (c) => !(c & bitMaskOffence); // offenceビット0でdefenseチェック。!x は x==0と同義
        const toPieceOriginal = (c) => (c & 0x7);

        const _copyBoardHand = (te) => {
            for (const k of num7to0) {
                myHand[handIndex(te, k)] = myHand[handIndex(te - 1, k)];
            }
            for (const y of num9to1) {
                for (const x of num9to1) {
                    _setBoard(te, x, y, _getBoard(te - 1, x, y));
                }
            }
        };

        const _getBoard = (te, x, y) => myBoard[boardIndex(te, x, y)];
        const _setBoard = (te, x, y, k) => (myBoard[boardIndex(te, x, y)] = k);

        const _incHand = (te, k) => {
            if (k != noPiece) {
                myHand[handIndex(te, k)]++;
            }
        };

        const _decHand = (te, k) => myHand[handIndex(te, k)]--;

        const _getHand = (te, k) => myHand[handIndex(te, k)];

        const _setIncDec = (incDec) => {
            const nextMoves = nowMoves + incDec;
            if ((nextMoves < 0) || (nextMoves > maxMoves) || !incDec) { // incDec == 0 と同義
                return false;
            }
            nowMoves = nextMoves;
            _fullDraw();
            return true;
        };

        const _newCsa = () => {
            if (fileName == "*") { // Ver.1.8 csaのインライン記入
                const inText = myElement.textContent;
                _loadCsaText(inText);
            } else {
                fetch(fileName, { cache: "no-store" }) // キャッシュno-store指定（Ver1.6）。エラー処理は、かなり雑
                    .then((response) => response.ok && response.text()) // response.ok==trueならばresponse.text()
                    .then((inText) => _loadCsaText(inText))
                    .catch((err) => console.error(err));
            }
        };

        const _loadCsaText = (inText) => {
            if (inText) { // if (inText != falsy) と同義
                context.textBaseline = "middle"; // 描画開始以降はtextBaselineはずっと"middle"
                _setCsaText(inText);
                _fullDraw();
                colorControls.setElement(myElement);
                setEvent(myElement);
            } else {
                console.log(fileName + " is not found");
            }
        };

        const _setCsaText = (inText) => {
            const inTextArray = inText.split(/\r\n|\n/);
            const sz = inTextArray.length; // 雑な配列領域確保
            myBoard = new Int8Array(sz * 9 * 9);
            myHand = new Int8Array(sz * pieceCount);
            myDestinationX = new Int8Array(sz);
            myDestinationY = new Int8Array(sz);

            for (const inLineOrg of inTextArray) {
                const inLine = inLineOrg.trimStart(); // Ver 1.8
                if (inLine.length < 6) {
                    continue;
                }
                const errLog = () => console.log("Err in " + fileName + ":" + inLine); // console.logへのエラー表示
                const number1char = (c) => { // １文字→数値変換
                    const r = Number(c);
                    if (!isTrustInFile && !r) { // !rは0またはNaNでtrueになる
                        errLog();
                        return 9; // エラー時に無難な値を返して処理継続
                    }
                    return r;
                };
                const getCode2char = (ch) => { // ２文字→駒コード変換
                    const r = getCodeByCSA(ch);
                    if (!isTrustInFile && (r < 0)) {
                        errLog();
                        return 7; // エラー時に無難な値を返して処理継続
                    }
                    return r;
                };
                const ch1 = inLine.substring(0, 1);
                if (ch1 == "P") {
                    const ch2 = inLine.substring(1, 2);
                    if (ch2 == "+") {
                        for (let xc = 4; xc < inLine.length; xc += 4) {
                            if (!isTrustInFile && (inLine.substring(xc - 2, xc) != "00")) {
                                errLog();
                                break;
                            }
                            _incHand(0, getCode2char(inLine.substring(xc, xc + 2)));
                        }
                    } else if (ch2 != "-") {
                        const y = number1char(ch2);
                        for (const x of num9to1) {
                            const xc = 29 - (x * 3);
                            _setBoard(0, x, y, getCodeByCSA(inLine.substring(xc, xc + 3))); // エラー処理省略
                        }
                    }
                } else if ((ch1 == "+") || (ch1 == "-")) {
                    maxMoves++;
                    _copyBoardHand(maxMoves);
                    const k = getCode2char(inLine.substring(5, 7));
                    if (inLine.substring(1, 3) == "00") { // 打つ
                        if (ch1 == "+") {
                            _decHand(maxMoves, k);
                        }
                    } else {
                        const fromX = number1char(inLine.substring(1, 2));
                        const fromY = number1char(inLine.substring(2, 3));
                        _setBoard(maxMoves, fromX, fromY, noPiece);
                    }
                    const destX = number1char(inLine.substring(3, 4));
                    const destY = number1char(inLine.substring(4, 5));
                    if (ch1 == "+") {
                        _incHand(maxMoves, _getBoard(maxMoves, destX, destY));
                        _setBoard(maxMoves, destX, destY, toOffence(k));
                    } else {
                        _setBoard(maxMoves, destX, destY, toDefense(k));
                    }
                    myDestinationX[maxMoves] = destX;
                    myDestinationY[maxMoves] = destY;
                }
            }

            const paramStart = strictNumber(elemDataset[dataStart]); // 開始局面の手数
            nowMoves = (paramStart > maxMoves ? maxMoves : paramStart);
            zeroGuide = elemDataset[dataZero] ?? currentScriptDataset[dataZero] ?? (maxMoves + "手詰");
        };

        const _fullDraw = () => { // 盤面と駒の描画
            // 盤面の描画
            context.clearRect(0, 0, myPositionSet.fullWidth, myPositionSet.fullHeight); // 全クリア
            context.textAlign = "center"; // textAlignは基本的には"center"

            if (nowMoves) { // 移動先の着色。if (nowMoves > 0) と同義
                context.fillStyle = colorControls.destinationColor;
                const destinationLeft = myPositionSet.gridPosX[myDestinationX[nowMoves]];
                const destinationTop = myPositionSet.gridPosY[myDestinationY[nowMoves] - 1];
                context.fillRect(destinationLeft, destinationTop, myPositionSet.fullGrid, myPositionSet.fullGrid);
            }
            context.strokeStyle = colorControls.fontColor;
            context.stroke(myPositionSet.pathBoardGrid); // fontColorで枠線の描画
            context.fillStyle = colorControls.fontColor; // 以降のfillは文字のみ

            // Watermarkの描画
            if (myWatermarkString) { // if (myWatermarkString != "") と同義
                context.setTransform(1, 1, -1, 1, 0, 0); // ななめ45度回転
                context.font = myPositionSet.watermarkFont;
                context.strokeStyle = "gray";
                context.strokeText(myWatermarkString, myPositionSet.watermarkX, 0, myPositionSet.watermarkWidth);
            }
            context.font = myPositionSet.font; // これ以降はデフォルトの色とフォント

            // 駒の描画、受け方
            context.setTransform(-1, 0, 0, -1, 0, 0); // 受け方は180度回転して負の座標に描画
            for (const y of num9to1) {
                for (const x of num9to1) {
                    const k = _getBoard(nowMoves, x, y);
                    if (isDefense(k)) {
                        context.fillText(pieceCharJ[k], -myPositionSet.fontPosX[x], -myPositionSet.fontPosY[y]); // 負の座標に描画
                    }
                }
            }

            // 駒の描画、攻め方
            context.scale(-1, -1); // 0度回転に戻す（直前が180度回転なので）
            for (const y of num9to1) {
                for (const x of num9to1) {
                    const k = _getBoard(nowMoves, x, y);
                    if (isOffence(k)) {
                        context.fillText(pieceCharJ[k], myPositionSet.fontPosX[x], myPositionSet.fontPosY[y]);
                    }
                }
            }

            // 座標数字の描画
            for (const xy of num9to1) {
                context.fillText(numberA[xy], myPositionSet.fontPosX[xy], myPositionSet.fontPosY[0]);
                context.fillText(numberJ[xy], myPositionSet.fontPosX[0], myPositionSet.fontPosY[xy]);
            }

            // 持駒などの描画
            let handStr = "";
            for (const k of num7to0) {
                const n = _getHand(nowMoves, k);
                if (n) { // if (n != 0) と同義
                    handStr += pieceCharJ[k];
                    if (n > 1) {
                        handStr += n;
                    }
                }
            }
            context.textAlign = "start";
            context.fillText("☗" + (handStr || "なし"), myPositionSet.handPosX, myPositionSet.handPosY, myPositionSet.handWidth);

            context.textAlign = "end";
            const guideStr = (nowMoves ? nowMoves + "手" : zeroGuide); // (nowMoves != 0) と同義
            context.fillText(guideStr, myPositionSet.movesPosX, myPositionSet.handPosY, myPositionSet.movesWidth);
        };

        const _incDecByPoint = (pt) => {
            const r = myElement.getBoundingClientRect();
            const x = pt.clientX - r.left;
            const y = pt.clientY - r.top;
            if ((y < myPositionSet.incDecTopEdge) || (y > myPositionSet.incDecBottomEdge)) { // 上下方向で盤外
                return 0;
            } else if ((x > myPositionSet.decLeftEdge) && (x < myPositionSet.decRightEdge) && nowMoves) { // 盤面左側。nowMoves > 0 と同義
                return -1;
            } else if ((x > myPositionSet.incLeftEdge) && (x < myPositionSet.incRightEdge) && (nowMoves < maxMoves)) { // 盤面右側
                return 1;
            } else {
                return 0;
            }
        };

        const _setWatermarkString = (watermarkString) => { // watermarkString==""なら呼ばれることはない
            if (!myWatermarkString) { // if(myWatermarkString == "")と同義
                myWatermarkString = watermarkString;
                _fullDraw();
            }
        };

        return {
            setIncDec: _setIncDec,
            newCsa: _newCsa,
            fullDraw: _fullDraw,
            incDecByPoint: _incDecByPoint,
            setWatermarkString: _setWatermarkString
        };
    };

    const getPositionSet = (sz) => {

        // デフォルトのフォント。textBaseline="middle"で上下幅がだいたい同じフォント
        const isFirefox = window.navigator.userAgent.toLowerCase().includes("firefox");
        const pxFontDefault = pxUnit + " " + (isFirefox ? "monospace" : "sans-serif");

        const positionSet = {
            fullGrid: sz * 2,
            fullWidth: sz * 22,
            fullHeight: sz * 22,
            handWidth: sz * 12,
            handPosX: sz,
            handPosY: sz * 21,
            movesPosX: sz * 20,
            movesWidth: sz * 6,
            fontPosX: new Int16Array(10),
            fontPosY: new Int16Array(10),
            gridPosX: new Int16Array(10),
            gridPosY: new Int16Array(10),
            decLeftEdge: sz,
            decRightEdge: sz * 9,
            incLeftEdge: sz * 11,
            incRightEdge: sz * 19,
            incDecTopEdge: sz * 2,
            incDecBottomEdge: sz * 20,
            font: Math.floor(sz * 1.4) + pxFontDefault,
            watermarkFont: (sz * 5) + pxFontDefault,
            watermarkX: Math.floor(sz * 10.5),
            watermarkWidth: sz * 14,
            pathBoardGrid: new Path2D() // 枠線のpathを格納するobject
        };
        const num9to0 = [...num9to1, 0];
        for (const xy of num9to0) {
            positionSet.fontPosX[xy] = sz * (20 - (2 * xy));
            positionSet.gridPosX[xy] = sz * (19 - (2 * xy)); // positionSet.fontPosX[xy] - sz;
            positionSet.gridPosY[xy] = sz * ((xy * 2) + 2);
            positionSet.fontPosY[xy] = sz * ((xy * 2) + 1); // positionSet.gridPosY[xy] - sz;
        }
        const tmpPathBoardGrid = positionSet.pathBoardGrid;
        for (const xy of num9to0) {
            tmpPathBoardGrid.moveTo(positionSet.gridPosX[xy], positionSet.gridPosY[0]);
            tmpPathBoardGrid.lineTo(positionSet.gridPosX[xy], positionSet.gridPosY[9]);
            tmpPathBoardGrid.moveTo(positionSet.gridPosX[0], positionSet.gridPosY[xy]);
            tmpPathBoardGrid.lineTo(positionSet.gridPosX[9], positionSet.gridPosY[xy]);
        }
        return positionSet;
    };

    const getMyPositionSet = (elem) => {
        const elemSize = getComputedStyle(elem);
        const sz = Math.floor(Math.min(
            22,
            elem.width / 22,
            elem.height / 22,
            parseFloat(elemSize.width) / 22,
            parseFloat(elemSize.height) / 22,
            /* (window.innerWidth - elem.offsetLeft) / 24, */
            docTarget.clientWidth / 24, // Ver.1.6 上位タグの横幅から算出
            window.innerHeight / 24
        ));
        return getPositionSet(sz);
    };

    const getAutoPositionSet = () => {
        // Ver.1.6 maxOffsetLeftの算出処理が不要になったため除去
        const sz = Math.floor(Math.min(
            22,
            /* window.innerWidth - maxOffsetLeft) / 24, */
            docTarget.clientWidth / 24, // Ver.1.6 上位タグの横幅から算出
            window.innerHeight / 24
        ));
        return getPositionSet(sz);
    };

    const initialProcess = () => {

        // fileパラメータ // Ver.1.5 dataFileのデフォルト設定
        const paramDataFile = currentScriptDataset[dataFile];
        const canvases = Array.from(docTarget.getElementsByTagName("canvas"));
        if (paramDataFile == "*") { // Ver.1.8 csaのインライン記入
            for (const elem of canvases) {
                elem.dataset[dataFile] ??= paramDataFile;
            }
        } else if (!isUndefined(paramDataFile)) {
            const startFile = ((paramDataFile || "1") + ".csa").split(".");
            const tmpStartNumber = strictNumber(startFile[0]);
            const fileExtension = "." + startFile[tmpStartNumber ? 1 : 0];
            let defaultDataFileNo = (tmpStartNumber || 1); // 未指定のときは１
            for (const elem of canvases) {
                elem.dataset[dataFile] ??= ((defaultDataFileNo++) + fileExtension);
            }
        }
        const legalElements = canvases.filter((elem) => elem.dataset[dataFile]); // data-file指定があるcanvasだけ処理対象とする
        const isAutoSizing = isSameSizeCanvas || legalElements.every(
            (elem) => {
                const elemSize = getComputedStyle(elem);
                return (elem.width == 300) && (elem.height == 150) && (elemSize.width == "300px") && (elemSize.height == "150px") // canvasのサイズが既定値のとき
            }
        );

        if (isAutoSizing) { // 全てのcanvasに同じサイズを設定
            const pSet = getAutoPositionSet();
            for (const elem of legalElements) {
                elem.option = candume(elem, pSet);
            }
        } else { // canvasごとにサイズを計算して設定
            for (const elem of legalElements) {
                const pSet = getMyPositionSet(elem);
                elem.option = candume(elem, pSet);
            }
        }
        for (const elem of legalElements) {
            elem.option.newCsa();
        }
    };

    initialProcess(); // <script>の配置をbody末尾以外でも可とするなら（一例） window.addEventListener("load", initialProcess)
})();
